/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.im.out.jbpm.utils;

import java.util.HashMap;
import java.util.Map;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Node;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.stp.im.BasicProperty;
import org.eclipse.stp.im.ImPackage;
import org.eclipse.stp.im.Process;
import org.eclipse.stp.im.Property;
import org.eclipse.stp.im.ServiceBinding;
import org.eclipse.stp.im.Step;
import org.eclipse.stp.im.StpIntermediateModel;
import org.eclipse.stp.im.constants.ImConstants;
import org.eclipse.stp.im.runtime.IRuntime;
import org.eclipse.stp.im.runtime.ImRuntimeActivator;
import org.spagic3.utils.ImPreprocessor;
import org.spagic3.utils.SpagicJBPMPreprocessor;
import org.spagic3.utils.SpagicUtils;

public class JBPMJetUtils {

	public static final String MEP_IN_ONLY = "http://www.w3.org/2004/08/wsdl/in-only";
	public static final String MEP_IN_OUT = "http://www.w3.org/2004/08/wsdl/in-out";
	public static final String XML_HEADER = "<\\?xml .*\\?>(\\r\\n)*";

	public static final String PROPERTY_MAP = "property-map".intern();
	public static final String MAP_PROPERTY_MAP = "map-property-map".intern();
	public static final String BEAN_PROPERTY_MAP = "bean-property-map".intern();
	public static final String LIST_PROPERTY_MAP = "list-property-map".intern();

	private static IRuntime runtime = null;
	private static Document jbpmSpecificDocument = null;

	public static Map<String, String> serviceBindingsTemplateMap = new HashMap<String, String>();
	public static Map<String, Document> extensionsDocument = new HashMap<String, Document>();

	public static final String REQUIRE_JNDI_DATASOURCE_PROPERTY_NAME = "RequireJndiDataSource";
	public static final String JNDI_DATASOURCE_PROPERTY_NAME = "JndiDataSource";

	public static String getRuntimeId() {
		return "runtime.jbpm";
	}

	public static void initJbpmConfiguration() throws RuntimeException {
		try {
			ImPackage imPackage = ImPackage.eINSTANCE;
			runtime = ImRuntimeActivator.getRuntime(getRuntimeId());
			String specificRuntimeXMLString = runtime.getRuntimeSpecific();

			jbpmSpecificDocument = DocumentHelper.parseText(specificRuntimeXMLString);

			Map<String, String> extensionsRuntimesSpecific = runtime.getRuntimeSpecificForExtension();
			if (extensionsRuntimesSpecific != null) {
				for (String key : extensionsRuntimesSpecific.keySet()) {
					extensionsDocument.put(key, DocumentHelper.parseText(extensionsRuntimesSpecific.get(key)));
				}
			}

		} catch (Exception e) {
			throw new RuntimeException(e);
		} finally {

		}
	}

	//
	// UTILITIES METHOD
	//
	private static Node selectNode(Document document, String xPath) {
		return document.selectSingleNode(xPath);
	}

	private static Node selectNode(String xPath) {
		Node node = selectNode(jbpmSpecificDocument, xPath);

		if (node != null)
			return node;
		for (String extKey : extensionsDocument.keySet()) {
			node = selectNode(extensionsDocument.get(extKey), xPath);
			if (node != null)
				return node;
		}
		return null;
	}

	public static String getVersionAsStringForProcess(Process aProcess) {
		Property processVersionProperty = aProcess.getProperties().get(ImConstants.PROCESS_VERSION);
		String version = null;
		if (processVersionProperty != null)
			version = (String) processVersionProperty.getValue();
		else
			version = "0";
		return version;
	}

	public static String getNamespaceURI(Process aProcess) {
		return SpagicUtils.getNameSpaceForProcess(aProcess);
	}

	public static String getNamespacePrefix(Process aProcess) {
		return SpagicUtils.getNameSpacePrefixForProcess(aProcess);
	}

	public static String getRuntimeNameSpacePrefix() {
		return runtime.getNsprefix();
	}

	public static String getRuntimeNameSpaceURI() {
		return runtime.getNsuri();
	}

	public static String getJetTemplateForServiceBinding(ServiceBinding serviceBinding) {
		String xPath = "/runtime-specific/code-generators/code-generator[@serviceBindingName='" + serviceBinding.getName()
				+ "']";
		Node node = selectNode(xPath);
		if (node == null)
			throw new RuntimeException(" Not found code-generator section for service binding [" + serviceBinding.getName()
					+ "]");
		return node.valueOf("@jetTemplate");
	}

	public static boolean isRequiredNamespaceMapForBinding(ServiceBinding serviceBinding) {
		String xPath = "/runtime-specific/code-generators/code-generator[@serviceBindingName='" + serviceBinding.getName()
				+ "']";
		Node node = selectNode(xPath);
		if (node == null)
			return false;
		String requireNamespaceMap = node.valueOf("@requireNamespaceMap");
		if (requireNamespaceMap != null && isValid(requireNamespaceMap))
			return Boolean.valueOf(requireNamespaceMap);
		else
			return false;
	}

	public static boolean isValid(String s) {
		return (s != null && s.trim().length() > 0);
	}

	public static String getRequiredPropertyValue(Step step, String imName) {
		Property aProperty = step.getProperties().get(imName);
		String valueInIm = aProperty != null ? aProperty.getValue().toString() : null;
		if (!JBPMJetUtils.isValid(valueInIm)) {
			throw new RuntimeException(" Property [" + imName + "] --> is required to generate code for Step ["
					+ step.getName() + "] with Binding [" + JBPMJetUtils.getRightStepBinding(step).getName() + "]");
		}
		return valueInIm;
	}

	public static String getOptionalPropertyValue(Step step, String imName, String defaultValue) {
		Property aProperty = step.getProperties().get(imName);
		String valueInIm = aProperty != null ? aProperty.getValue().toString() : defaultValue;
		return valueInIm;
	}

	/**
	 * 
	 * @param s
	 * @return the right service binding for the runtime we're using during the
	 *         transformation
	 */
	public static ServiceBinding getRightStepBinding(Step s) {
		String runtimeID = getRuntimeId();
		Property runtimPropertyForServiceBinding = null;
		for (ServiceBinding sb : s.getStepbindings()) {
			runtimPropertyForServiceBinding = sb.getProperties().get(ImConstants.IM_STEP_BINDING_RUNTIME_ID);
			if (runtimPropertyForServiceBinding != null) {
				if (runtimPropertyForServiceBinding.getValue().equals(runtimeID)) {
					return sb;
				}
			} else {
				// LogUtils.warn("ServiceBinding["+sb.getName()+"] -> for Step["+s.getName()+"] does not have a property called "+
				// ImConstants.IM_POOL_RUNTIME_ID);
			}
		}
		// LogUtils.warn("Not found for Step["+s.getName()+"] a binding for runtime "+
		// ImConstants.IM_POOL_RUNTIME_ID);
		return null;
	}

	public static String getRelativeFileName(String original) {
		String relPath = original;
		if (original.indexOf("/") > 0)
			relPath = relPath.substring(original.lastIndexOf("/") + 1);
		if (original.indexOf("\\") > 0)
			relPath = relPath.substring(original.lastIndexOf("\\") + 1);
		return relPath;
	}

	

	public static void jbpmPreprocessor(IFile imFile, StpIntermediateModel aStpIntermediateModel) {
		ImPreprocessor imPreprocessor = new SpagicJBPMPreprocessor(getRuntimeId());
		imPreprocessor.process(imFile, aStpIntermediateModel);
	}

	public static boolean existFileInProject(String projectName, String wsdlFile) {
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot root = workspace.getRoot();
		IProject proj = root.getProject(projectName);
		IFile file = proj.getFile(wsdlFile);
		return file.exists();
	}

	/**
	 * Removes the xml header.
	 * 
	 * @param xmlString
	 *          the xml string
	 * 
	 * @return the string without the XML header
	 */
	public static String removeXMLHeader(String xmlString) {
		return xmlString.replaceFirst(XML_HEADER, "");
	}

	public static String escapeForXML(String s) {
		String result = s.replaceAll("<", "&lt;");
		return result;
	}

	public static Map<String, String> getMapVariable(Step step, String mapVariableName, String key, String value) {
		Map<String, String> result = new HashMap<String, String>();
		Map<String, Map<String, String>> mapValues = getMapValue(step, mapVariableName);
		for (String str : mapValues.keySet()) {
			Map<String, String> values = mapValues.get(str);
			String vKey = values.get(key);
			String vValue = values.get(value);
			result.put(vKey, vValue);
		}
		return result;
	}

	public static Map<String, Map<String, String>> getMapValue(Step aStep, String imMapName) {
		Map<String, Map<String, String>> mapProperties = new HashMap<String, Map<String, String>>();
		String idxMapPropertyString = null;
		Map<String, String> entryValues = null;
		String displayName = null;
		Property inStepProperty = null;
		for (String key : aStep.getProperties().keySet()) {
			if (key.indexOf("[") <= 0) // This property is not related to a map
																	// property
				continue;
			if (!key.startsWith(imMapName)) // This property is related to another map
				continue;
			inStepProperty = aStep.getProperties().get(key);
			idxMapPropertyString = key.substring(key.indexOf("[") + 1, key.indexOf("]"));
			displayName = key.substring(key.indexOf("]") + 2);
			entryValues = mapProperties.get(idxMapPropertyString);
			if (entryValues == null)
				entryValues = new HashMap<String, String>();
			entryValues.put(displayName, (String) inStepProperty.getValue());
			mapProperties.put(idxMapPropertyString, entryValues);
		}
		return mapProperties;
	}

	public static boolean hasTimer(Step aStep) {
		BasicProperty timerType = (BasicProperty) aStep.getProperties().get("TimerType");
		boolean retValue = false;
		if (timerType != null) {
			if (timerType.getValue().equalsIgnoreCase("RelativeExpirationTimer")
					|| timerType.getValue().equalsIgnoreCase("AbsoluteExpirationTimer")) {
				retValue = true;
			}
		}
		return retValue;
	}

	public static boolean isAbsoluteTimer(Step aStep) {
		BasicProperty timerType = (BasicProperty) aStep.getProperties().get("TimerType");
		if (timerType != null) {
			return timerType.getValue().equalsIgnoreCase("AbsoluteExpirationTimer");
		}
		return false;
	}
}
