package org.spagic3.im.out.wsdl.utils;


import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.charset.CharacterCodingException;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CodingErrorAction;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;

/**
 *  FileMemoryMapper è una collezione di utilities statiche per mappare i file.
 *
 *
 * Created: Thu Oct  4 10:33:29 2007
 *
 * @author <a href="mailto:saint@eng.it">Gian Uberto Lauri</a>
 * @version $Revision: 1.2 $
 */
public class FileMemoryMapper {

	/**
	 * <code>extractStream</code> crea uno InputStream partendo da un IFile
	 *
	 * @param file an <code>IFile</code> l'IFile da cui si parte
	 * @return a <code>FileInputStream</code> lo stream che viene creato
	 * @exception FileMemoryMapperException Se non si trova il file
	 */
	public final static FileInputStream extractStream(IFile file)
	throws FileMemoryMapperException {
		try {
			File phFile = file.getRawLocation().toFile();
			return new FileInputStream( phFile );
		} catch (FileNotFoundException e) {
			throw new FileMemoryMapperException(e,ErrorCodes.UNEXPECTED_IO_ERROR);
		}
	}

	/**
	 * <code>makeDecoder</code> crea l'oggetto Decoder che traduce i byte
	 * del file in char 
	 *
	 * @param file an <code>IFile</code> lo IFile (da cui si estrae il
	 * charset usato)
	 * @return a <code>CharsetDecoder</code> il CharsetDecoder creato
	 * @exception FileMemoryMapperException se viene sollevata una
	 * CoreException da Eclipse
	 */
	public final static CharsetDecoder makeDecoder(IFile file)
	throws FileMemoryMapperException {

		Charset charset=null;

		try {
			String charsetName = file.getCharset();
			charset = Charset.forName(charsetName);
		} catch (CoreException e) {
			throw new FileMemoryMapperException(e,ErrorCodes.JDT_ERROR);
		}
		return charset.newDecoder();
	}

	/**
	 * <code>mapStream</code> Mappa in memora l'InputStream usando il
	 * CharsetDecoder per tradurre i byte in char
	 *
	 * @param fis a <code>FileInputStream</code> il FileInputStream di
	 * partenza
	 * @param decoder a <code>CharsetDecoder</code> il CharsetDecoder usato
	 * per tradurre i byte in char
	 * @return a <code>CharSequence</code> il file mappato in memoria
	 * @exception FileMemoryMapperException Se fallisce la mappatura a
	 * causa di un IOException o di un problema di decoding dei byte
	 */
	public final static MappedStream mapStream(FileInputStream fis,
											   CharsetDecoder decoder)
	throws FileMemoryMapperException {
		FileChannel fic = fis.getChannel();

		MappedByteBuffer bb;
		int size;
		try {
			size = (int)fic.size();
			bb = fic.map(FileChannel.MapMode.READ_ONLY,
						 0, size);
		} catch (IOException e) {
			throw new FileMemoryMapperException(e, ErrorCodes.UNEXPECTED_IO_ERROR);
		}

		decoder.onMalformedInput(CodingErrorAction.REPLACE);

		try {
			
			return new Mappato(fis,decoder.decode(bb));
		} catch (CharacterCodingException e) {
			throw new FileMemoryMapperException(e, ErrorCodes.UNEXPECTED_IO_ERROR);
		}
	}

	/**
	 * <code>safeClose</code> chiude in modo pulito (per il chiamante) il
	 * file mappato in memoria
	 *
	 * @param fis a <code>FileInputStream</code> l'InputStreeam da chiudere
	 * @exception FileMemoryMapperException l'InputStreeam da chiudere
	 * IOException
	 */
	public final static void safeClose(MappedStream fis)
    throws FileMemoryMapperException {
		try {
			if (fis != null) {
				fis.close();
			}
		} catch (IOException e) {
			throw new FileMemoryMapperException(e,ErrorCodes.JDT_ERROR);
		}
	}

}

class Mappato implements MappedStream{
	FileInputStream file;
	CharSequence seq;
	
	public Mappato(FileInputStream file, CharSequence seq) {
		this.file = file;
		this.seq = seq;
	}

	public void close() throws IOException {
		seq = null;
		// Qualche cosina per fare si che la sequence venga finalizzata.
		System.gc();
		Thread.yield();
		if(file != null) {
			file.getChannel().close();
			file.close();
		}
	}

	public CharSequence getMappedStream() {
		return seq;
	}
	
}

