/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
 **/
package org.spagic3.ui.deploymentservice.wizards;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.net.URI;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IWorkbench;
import org.spagic3.service.model.ServiceModelHelper;
import org.spagic3.service.model.ServiceModelHelper.DeploySchemaHelper;
import org.spagic3.ui.deploymentservice.Activator;
import org.spagic3.ui.deploymentservice.actions.PreparePackagingArtifacts;
import org.spagic3.utils.SpagicUtilsActivator;
import org.spagic3.utils.preference.SpagicPreferenceConstants;

public class ServiceDeploymentWizard extends Wizard implements IWizard {

	private ServiceDeploymentChoicePage choicePage;
	private ServiceDeploymentPage servicePage;
	private ProcessDeploymentPage processPage;
	private ISelection selection;
	private boolean showProjectList;

	public ServiceDeploymentWizard(boolean showProjectList) {
		super();
		setNeedsProgressMonitor(true);
		setHelpAvailable(false);
		this.showProjectList = showProjectList;
	}

	public void addPages() {
		choicePage = new ServiceDeploymentChoicePage(selection, showProjectList);
		addPage(choicePage);
		servicePage = new ServiceDeploymentPage(selection);
		addPage(servicePage);
		processPage = new ProcessDeploymentPage(selection);
		addPage(processPage);
	}

	public IWizardPage getNextPage(IWizardPage page) {
		if (page == choicePage) {
			return choicePage.isProcessDeploy() ? processPage : servicePage;
		} else if (page == servicePage) {
			return null;
		} else if (page == processPage) {
			return servicePage;
		}
		return null;
	}

	public IWizardPage getPreviousPage(IWizardPage page) {
		if (page == choicePage) {
			return null;
		} else if (page == servicePage) {
			return choicePage.isProcessDeploy() ? processPage : choicePage;
		} else if (page == processPage) {
			return choicePage;
		}
		return null;
	}

	public boolean isProcessDeploy() {
		return choicePage.isProcessDeploy();
	}

	public boolean canFinish() {
		IWizardPage page = getContainer().getCurrentPage();
		if (page == servicePage) {
			return servicePage.isPageComplete();
		}
		return false;
	}

	public boolean performFinish() {
		
		
		boolean processesOK = deployProcesses();
		if (!processesOK){
			MessageDialog.openError(getShell(), "Error In Process Deployment", "The wizard will be stopped");
			return this.performCancel();
		}
		boolean servicesOK = deployServices();
		if (!servicesOK){
			MessageDialog.openError(getShell(), "Error In Services Deployment", "The wizard will be stopped");
			return this.performCancel();
		}
		return processesOK && servicesOK;
	}
	
	
	
	
	protected boolean runOperation(IRunnableWithProgress op) {
		try {
			getContainer().run(true, false, op);
			return true;
		} catch (InterruptedException e) {
			return false;
		} catch (InvocationTargetException e) {
			Throwable realException = e.getTargetException();
			MessageDialog.openError(getShell(), "Error", realException
					.getMessage());
			return false;
		}
	}
	
	
	protected List<String> getSelectedProcessesPoolNames(){
		final List<URI> selectedURIs = servicePage.getSelectedURIs();
		
		List<String> processNames = new ArrayList<String>();
		ServiceInfoRepository repo = Activator.getDefault().getServiceInfoRepository(servicePage.getProject());
		for (URI uri : selectedURIs){
			ServiceInfo info = repo.findServiceInfoByURI(uri);
			if (info.getProcess() != null){
				processNames.add(info.getProcess());
			}
		}
		return processNames;
	
	}
	protected boolean deployProcesses(){
		List<URI> processRelatedURIs = new ArrayList<URI>();
		
		List<String> selectedProcessNames = getSelectedProcessesPoolNames();
		if (isProcessDeploy() && processPage.isDeployJPDLs()) {
			
			for (URI bpmnURI : processPage.getSelectedBpmnURIs()) {
				PreparePackagingArtifacts packagingArtifacts = new PreparePackagingArtifacts();
				final IFile[] bpmnFiles = ResourcesPlugin.getWorkspace().getRoot()
							.findFilesForLocationURI(bpmnURI);
				
				if (bpmnFiles != null && bpmnFiles[0] != null) {
					
					packagingArtifacts.setBpmnFile(bpmnFiles[0]);
					packagingArtifacts.selectedProcessNames(selectedProcessNames);
					packagingArtifacts.doPackaging(processRelatedURIs);
					
				}
				if (packagingArtifacts.isErrors())
					return false;
			}
		}
		
		final List<URI> fprocessRelatedURIs = processRelatedURIs;
		
		
		IRunnableWithProgress opDeployPackaging = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException {
				try {
					doFinishPackageDeployment(fprocessRelatedURIs, monitor);
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} finally {
					monitor.done();
				}
			}
		};
		
		return runOperation(opDeployPackaging);
	}
	
	protected boolean deployServices(){
		final List<URI> selectedURIs = servicePage.getSelectedURIs();
		final List<URI> resourceURIs = servicePage.getSelectedDependencyURIs();
		
		
			
			IRunnableWithProgress op = new IRunnableWithProgress() {
				public void run(IProgressMonitor monitor) throws InvocationTargetException {
					try {
						doFinish(resourceURIs, selectedURIs, monitor);
					} catch (CoreException e) {
						throw new InvocationTargetException(e);
					} finally {
						monitor.done();
					}
				}
			};
			
			return runOperation(op);
			
	}

	private void copyFile(File srcFile, File dstFile) {
		try {
			final FileChannel srcChannel = new FileInputStream(srcFile)
					.getChannel();
			final FileChannel dstChannel = new FileOutputStream(dstFile)
					.getChannel();
			dstChannel.transferFrom(srcChannel, 0, srcChannel.size());
			srcChannel.close();
			dstChannel.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private void doFinish(List<URI> resourceURIs, List<URI> selectedURIs,
			IProgressMonitor monitor) throws CoreException {
		monitor.beginTask("Deploying services... ", 1);
		IPreferenceStore preferences = SpagicUtilsActivator.getDefault()
				.getPreferenceStore();

		String dstDirName = preferences
				.getString(SpagicPreferenceConstants.P_DEPLOY_SERVICE_TARGET_DIR);
		if (dstDirName == null || "".equals(dstDirName)) {
			getShell().getDisplay().asyncExec(new Runnable() {
				public void run() {
					MessageDialog.openError(getShell(), "Error",
							"Deployment Path is empty");
				}
			});
			monitor.worked(1);
			return;
		}

		File dstDir = new File(dstDirName);
		if (!dstDir.exists()) {

			getShell().getDisplay().asyncExec(new Runnable() {
				public void run() {
					MessageDialog.openError(getShell(), "Error",
							"file not found");
				}
			});
			monitor.worked(1);
			return;
		}

		List<URI> toCopyURIs = new ArrayList<URI>();
		toCopyURIs.addAll(resourceURIs);
		toCopyURIs.addAll(selectedURIs);
		ServiceModelHelper serviceModelHelper;
		try {
			serviceModelHelper = new ServiceModelHelper();
		} catch (Exception e) {
			// e.printStackTrace();
			throw new CoreException(new Status(IStatus.ERROR,
					Activator.PLUGIN_ID, IStatus.ERROR, e.getMessage(), null));
		}
		for (URI uri : toCopyURIs) {
			final File srcFile = new File(uri);
			final String filename = srcFile.getName();
			int dotIndex = filename.lastIndexOf(".");
			final String extension = dotIndex == -1 ? "" : filename
					.substring(dotIndex + 1);
			final DeploySchemaHelper schemaHelper = serviceModelHelper != null ? serviceModelHelper
					.getDeploySchemaByExtension(extension)
					: null;
			final File dstSubDir = schemaHelper != null ? new File(dstDir,
					schemaHelper.getPath()) : dstDir;
			if (!dstSubDir.exists()) {
				dstSubDir.mkdirs();
			}
			final File dstFile = new File(dstSubDir, filename);
			if (dstFile != null) {
				copyFile(srcFile, dstFile);
			}
		}
		monitor.worked(1);
	}
	
	
	private void doFinishPackageDeployment(List<URI> processRelatedURIs,
			IProgressMonitor monitor) throws CoreException {
		monitor.beginTask("Deploying Process Files and Related Resources... ", 1);
		IPreferenceStore preferences = SpagicUtilsActivator.getDefault()
				.getPreferenceStore();

		String dstDirName = preferences
				.getString(SpagicPreferenceConstants.P_DEPLOY_SERVICE_TARGET_DIR);
		
		dstDirName += File.separator + "processes";
		
		if (dstDirName == null || "".equals(dstDirName)) {
			getShell().getDisplay().asyncExec(new Runnable() {
				public void run() {
					MessageDialog.openError(getShell(), "Error",
							"Path is empty");
				}
			});
			monitor.worked(1);
			return;
		}

		File dstDir = new File(dstDirName);
		if (!dstDir.exists()) {

			dstDir.mkdirs();
		}

		List<URI> toCopyURIs = new ArrayList<URI>();
		toCopyURIs.addAll(processRelatedURIs);
		
		
		for (URI uri : toCopyURIs) {
			final File srcFile = new File(uri);
			final String filename = srcFile.getName();
			int dotIndex = filename.lastIndexOf(".");
			final String extension = dotIndex == -1 ? "" : filename
					.substring(dotIndex + 1);
			
			
			final File dstSubDir = new File(dstDir,extension);
			if (!dstSubDir.exists()) {
				dstSubDir.mkdirs();
			}
			final File dstFile = new File(dstSubDir, filename);
			if (dstFile != null) {
				copyFile(srcFile, dstFile);
			}
		}
		monitor.worked(1);
	}

	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
	}
}