/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.ui.metadataeditor.editors;

import java.util.List;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.editor.FormPage;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.part.FileEditorInput;
import org.spagic3.ui.metadataeditor.Activator;
import org.spagic3.ui.metadataeditor.MetadataEditorInput;
import org.spagic3.ui.metadataeditor.model.IModel;
import org.spagic3.ui.metadataeditor.model.IModelListener;
import org.spagic3.ui.metadataeditor.model.ModelChangeType;
import org.spagic3.ui.metadataeditor.model.ModelHelper;

public class MetadataEditor extends org.eclipse.ui.forms.editor.FormEditor implements IResourceChangeListener, IModelListener {

	
	private static String OUT_OF_DATE_XML_PREFIX = " - out of date - \n";
	private XMLEditor xmlEditor;
	private FormPage formEditor;
	private int xmlEditorPageIndex;
	private int formEditorPageIndex;
	
	private MetadataEditorInput metadataEditorInput;
	private ModelHelper modelHelper;
	private IModel model;
	
	public MetadataEditor() {
		super();
		ResourcesPlugin.getWorkspace().addResourceChangeListener(this);
	}
	
	public void init(IEditorSite site, IEditorInput editorInput)
		throws PartInitException {
		if (!(editorInput instanceof IFileEditorInput))
			throw new PartInitException("Invalid Input: Must be IFileEditorInput");
		super.init(site, editorInput);
		metadataEditorInput = new MetadataEditorInput((IFileEditorInput) editorInput);
		modelHelper = new ModelHelper();
	}

	public MetadataEditorInput getMetadataEditorInput() {
		return metadataEditorInput;
	}

	public void dispose() {
		ResourcesPlugin.getWorkspace().removeResourceChangeListener(this);
		super.dispose();
	}
	
	protected FormToolkit createToolkit(Display display) {
		// Create a toolkit that shares colors between editors.
		return new FormToolkit(Activator.getDefault().getFormColors(display));
	}
	
	public boolean isSaveAsAllowed() {
		return true;
	}
	
	public boolean isDirty() {
		return xmlEditor.isDirty();
	}
	
	private void warnForMandatoryFields() {
		List<String> warningFields = modelHelper.warningFields(model);
		if (!warningFields.isEmpty() ) {
			StringBuffer message = new StringBuffer("\tsave operation will complete with the following warning(s).\n");
			message.append("\nWarnings:\n");
			for (String warning : warningFields) {
				message.append("\t" + warning + "\n");
			}
			ErrorDialog.openError(
					getSite().getShell(),
					"Warning",
					"Some warning for empty mandatory fields or out of range values.",
					new Status(IStatus.WARNING, Activator.PLUGIN_ID,
							IStatus.WARNING, message.toString(), null));
		}
	}


	public void doSave(IProgressMonitor monitor) {
		if (isXMLEditorTextOutOfDate()) {
			updateXML();
		}
		warnForMandatoryFields();
		getEditor(xmlEditorPageIndex).doSave(monitor);
	}

	public void doSaveAs() {
		if (isXMLEditorTextOutOfDate()) {
			updateXML();
		}
		warnForMandatoryFields();
		IEditorPart editor = getEditor(xmlEditorPageIndex);
		editor.doSaveAs();
		setInput(new MetadataEditorInput((IFileEditorInput) editor.getEditorInput()));
		updateModel();
		updateTitle();
	}

	public void gotoMarker(IMarker marker) {
		setActivePage(xmlEditorPageIndex);
		IDE.gotoMarker(getEditor(xmlEditorPageIndex), marker);
	}
	
	protected void addPages() {
		createXMLEditorPage();
		createFormEditorPage();
		updateTitle();
	}
	
	private void createXMLEditorPage() {
		try {
			xmlEditor = new XMLEditor();
//			xmlEditorPageIndex = addPage(xmlEditor, 
//					((MetadataEditorInput) getEditorInput()).getFileEditorInput());
			xmlEditorPageIndex = addPage(xmlEditor, getEditorInput());
			setPageText(xmlEditorPageIndex, "XML");
		} catch (PartInitException e) {
			ErrorDialog.openError(
				getSite().getShell(),
				"Error creating nested xml editor",
				null,
				e.getStatus());
		}
	}
	
	private String getXMLEditorText() {
		return xmlEditor.getDocumentProvider()
				.getDocument(xmlEditor.getEditorInput()).get();
	}

	private void setXMLEditorText(String xmlText) {
		xmlEditor.getDocumentProvider()
				.getDocument(xmlEditor.getEditorInput())
						.set(xmlText);
	}

	private void updateModel() {
		try {
			IModel oldModel = model;

//			if (getXMLEditorText() == null || "".equals(getXMLEditorText())) {
//				setXMLEditorText("<metadatadefinition/>");
//			}
			
			model = modelHelper.buildFromXML(getXMLEditorText());
//			((MetadataEditorInput) getEditorInput()).setModel(model);
			metadataEditorInput.setModel(model);
			if (oldModel != null) {
				model.addListeners(oldModel);
			} else {
				model.addModelListener(this);
			}
			model.fireModelChanged(new Object[] {model, oldModel}, ModelChangeType.CHANGE_NEW);
		} catch (Exception e) {
			ErrorDialog.openError(
					getSite().getShell(),
					"Error updating model: maybe not an xml file",
					null,
					new Status(IStatus.ERROR, Activator.PLUGIN_ID,
							IStatus.ERROR, e.getMessage(), e));
		}
	}

	public void modelChanged(Object[] objects, ModelChangeType type) {
		if (type != ModelChangeType.CHANGE_NEW) {
			markXMLEditorTextOutOfDate();
		}
	}

	private void markXMLEditorTextOutOfDate() {
		setXMLEditorText(OUT_OF_DATE_XML_PREFIX + getXMLEditorText());
	}
	
	private boolean isXMLEditorTextOutOfDate() {
		return getXMLEditorText().startsWith(OUT_OF_DATE_XML_PREFIX);
	}
	
	private void updateXML() {
		String xmlFromModel = modelHelper.asXML(model);
		if (xmlFromModel != null && !xmlFromModel.equals(getXMLEditorText())) {
			setXMLEditorText(xmlFromModel);
		}
	}
	
	private void createFormEditorPage() {
		try {
			updateModel();
			formEditor = new MasterDetailsPage(this);
			formEditorPageIndex = addPage(formEditor);
			setPageText(formEditorPageIndex, "Metadata");
			setActivePage(formEditorPageIndex);
		} catch (PartInitException e) {
			ErrorDialog.openError(
					getSite().getShell(),
					"Error creating nested metadata editor",
					null,
					e.getStatus());
		} catch (Exception e) {
			ErrorDialog.openError(
					getSite().getShell(),
					"Error creating nested metadata editor model",
					null,
					new Status(IStatus.ERROR, Activator.PLUGIN_ID,
							IStatus.ERROR, e.getMessage(), e));
		}
	}
	
	protected void pageChange(int pageIndex) {
		super.pageChange(pageIndex);
		if (pageIndex == xmlEditorPageIndex) {
			updateXML();
		} else if (pageIndex == formEditorPageIndex) {
			if (getXMLEditorText() != null 
					&& !getXMLEditorText().equals(modelHelper.asXML(model))) {
				updateModel();
			}
		}
	}
	
	public void setFocus() {
		int active = getActivePage();
		if (active == xmlEditorPageIndex) {
			xmlEditor.setFocus();
		} else if (active == formEditorPageIndex) {
			formEditor.setFocus();
		}
	}
	
	void updateTitle() {
		IEditorInput input = getEditorInput();
		setPartName(input.getName());
		setTitleToolTip(input.getToolTipText());
	}
	
	public void resourceChanged(final IResourceChangeEvent event) {
		if(event.getType() == IResourceChangeEvent.PRE_CLOSE) {
			Display.getDefault().asyncExec(new Runnable() {
				public void run() {
					IWorkbenchPage[] pages = getSite().getWorkbenchWindow().getPages();
					for (int i = 0; i < pages.length; i++) {
						if(((FileEditorInput)xmlEditor.getEditorInput()).getFile().getProject().equals(event.getResource())) {
							IEditorPart editorPart = pages[i].findEditor(xmlEditor.getEditorInput());
							pages[i].closeEditor(editorPart, true);
						}
					}
				}            
			});
		}
	}

}
