/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 **/
package org.spagic3.ui.serviceeditor.editors;

import java.util.List;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.editor.IFormPage;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.part.FileEditorInput;
import org.spagic3.ui.serviceeditor.Activator;
import org.spagic3.ui.serviceeditor.model.IServiceModel;
import org.spagic3.ui.serviceeditor.model.ServiceModelHelper;

public class ServiceEditor extends FormEditor implements IResourceChangeListener {

	// changed to default, a bit harder than protected.
	XMLEditor xmlEditor;
	FormModelPage formPage;
	int xmlEditorPageIndex;
	int formPageIndex;

	ServiceModelHelper helper;
	IServiceModel model;

	public ServiceEditor() {
		super();
		ResourcesPlugin.getWorkspace().addResourceChangeListener(this);
	}

	public void init(IEditorSite site, IEditorInput editorInput)
	throws PartInitException {
		if (!(editorInput instanceof IFileEditorInput))
			throw new PartInitException("Invalid Input: Must be IFileEditorInput");
		super.init(site, editorInput);
	}

	public void dispose() {
		ResourcesPlugin.getWorkspace().removeResourceChangeListener(this);
		super.dispose();
	}

	protected FormToolkit createToolkit(Display display) {
		// Create a toolkit that shares colors between editors.
		return new FormToolkit(Activator.getDefault().getFormColors(
				display));
	}

	public boolean isSaveAsAllowed() {
		return true;
	}

	private void warnForMandatoryFields() {
		List<String> emptyMandatoryProperties
		= helper.getEmptyMandatoryProperties(model);
		List<String> emptyMandatoryMapProperties 
		= helper.getEmptyMandatoryMapProperties(model);
		if (!emptyMandatoryProperties.isEmpty() 
				|| !emptyMandatoryMapProperties.isEmpty()) {
			StringBuffer message = new StringBuffer("\tthe following fields are mandatory but empty.\n");
			if (!emptyMandatoryProperties.isEmpty()) {
				message.append("\nProperties:\n");
				for (String property : emptyMandatoryProperties) {
					message.append("\t" + property + "\n");
				}
			}
			if (!emptyMandatoryMapProperties.isEmpty()) {
				message.append("\nMap properties:\n");
				for (String mapProperty : emptyMandatoryMapProperties) {
					message.append("\t" + mapProperty + "\n");
				}
			}
			ErrorDialog.openError(
					getSite().getShell(),
					"Mandatory fields warning",
					"Some mandatory fields are empty.",
					new Status(IStatus.WARNING, Activator.PLUGIN_ID,
							IStatus.WARNING, message.toString(), null));
		}
	}

	@Override
	public boolean isDirty() {
		return xmlEditor.isDirty();
	}

//	private AtomicBoolean saving = new AtomicBoolean(false);
	public void doSave(IProgressMonitor monitor) {
		if (formPage.isModelDirty()) {
			formPage.refreshModel();
		}
		warnForMandatoryFields();
//		saving.set(true);
		getEditor(xmlEditorPageIndex).doSave(monitor);

	}


	public void doSaveAs() {
		if (formPage.isModelDirty()) {
			formPage.refreshModel();
		}
		warnForMandatoryFields();
		IEditorPart editor = getEditor(xmlEditorPageIndex);
//		saving.set(true);
		editor.doSaveAs();
		//		setPageText(xmlEditorPageIndex, editor.getTitle());
		setInput(editor.getEditorInput());
		updateTitle();
	}

	public void gotoMarker(IMarker marker) {
		setActivePage(xmlEditorPageIndex);
		IDE.gotoMarker(getEditor(xmlEditorPageIndex), marker);
	}

	protected void addPages() {
		createXMLEditorPage();
		createFormEditorPage();
		updateTitle();
	}

	private void createXMLEditorPage() {
		try {
			xmlEditor = new XMLEditor();
			xmlEditorPageIndex = addPage(xmlEditor, getEditorInput());
			setPageText(xmlEditorPageIndex, "XML");
		} catch (PartInitException e) {
			ErrorDialog.openError(
					getSite().getShell(),
					"Error creating nested xml editor",
					null,
					e.getStatus());
		}
	}

	void refreshModel() {
		helper.applyRules(model);
		refreshXML();

		Display.getDefault().asyncExec(new Runnable() {
			public void run() {
				updateFormPage();
			}
		});
	}

	private void updateFormPage() {
		try {
			//			formPage.removeFocusListeners();
			formPage = new FormModelPage(formPage);
			addPage(formPageIndex, formPage);
			setPageText(formPageIndex, "Form");
			setActivePage(formPageIndex);

			formPage.copyFormStatus((FormModelPage) pages.get(formPageIndex + 1));
			removePage(formPageIndex + 1);

		} catch (PartInitException e) {
			ErrorDialog.openError(
					getSite().getShell(),
					"Error creating nested form editor",
					null,
					e.getStatus());
		}
	}

	void refreshXML() {
		String xmlFromModel = helper.asXML(model);
		String actualXML = xmlEditor.getDocumentProvider()
		.getDocument(xmlEditor.getEditorInput()).get();
		if (!xmlFromModel.equals(actualXML)) {
			xmlEditor.getDocumentProvider()
			.getDocument(xmlEditor.getEditorInput())
			.set(xmlFromModel);
		}
	}

	private void createFormEditorPage() {
		try {
			String xmlEditorText =
				xmlEditor.getDocumentProvider()
				.getDocument(xmlEditor.getEditorInput())
				.get();

			//parse xml and create model
			helper = new ServiceModelHelper();
			model = helper.createModel(xmlEditorText);

			//create form from model
			formPage = new FormModelPage(this);
			formPageIndex = addPage(formPage);
			setPageText(formPageIndex, "Form");
			setActivePage(formPageIndex);
		} catch (PartInitException e) {
			ErrorDialog.openError(
					getSite().getShell(),
					"Error creating nested form editor",
					null,
					e.getStatus());
		} catch (Exception e) {
			ErrorDialog.openError(
					getSite().getShell(),
					"Error creating nested form editor model",
					null,
					new Status(IStatus.ERROR, Activator.PLUGIN_ID,
							IStatus.ERROR, e.getMessage(), e));
		}
	}

	protected void pageChange(int pageIndex) {
		super.pageChange(pageIndex);
		if (pageIndex == xmlEditorPageIndex) {
			refreshXML();
		} else if (pageIndex == formPageIndex) {
		}
	}

	public void setFocus() {
		int active = getActivePage();
		if (active == xmlEditorPageIndex) {
			xmlEditor.setFocus();
		} else if (active == formPageIndex) {
			formPage.setFocus();
		}
	}

	void updateTitle() {
		IEditorInput input = getEditorInput();
		setPartName(input.getName());
		setTitleToolTip(input.getToolTipText());
	}

	public void resourceChanged(final IResourceChangeEvent event) {
// Disabled, not properly working.
//
//		if(event.getType() == IResourceChangeEvent.POST_CHANGE) {
//			if (saving.get()) 
//				saving.set(false);
//			else
//				xmlEditor.doRevertToSaved();
//		}
		if(event.getType() == IResourceChangeEvent.PRE_CLOSE) {
			Display.getDefault().asyncExec(new Runnable() {
				public void run() {
					IWorkbenchPage[] pages = getSite().getWorkbenchWindow().getPages();
					for (int i = 0; i < pages.length; i++) {
						if(((FileEditorInput)xmlEditor.getEditorInput()).getFile().getProject().equals(event.getResource())) {
							IEditorPart editorPart = pages[i].findEditor(xmlEditor.getEditorInput());
							pages[i].closeEditor(editorPart, true);
						}
					}
				}            
			});
		}
	}

	public ServiceModelHelper getHelper() {
		return helper;
	}

	public IServiceModel getModel() {
		return model;
	}

	public FormModelPage getFormPage() {
		return formPage;
	}

	@Override
	public IFormPage findPage(String pageId) {
		if("Form".equals(pageId))
			return formPage;
		else
			return null;
	}


	// used by the GenerateWSDLAction.run() to
	// update editor contents... Attempts to call
	// this method from the resourceChanged() one failed
	// during the execution of the 'refresh' command;
	public void update(String s, IProgressMonitor monitor) {
				xmlEditor.getDocumentProvider()
				.getDocument(xmlEditor.getEditorInput())
				.set(s);
		try {
			model = helper.createModel(s);
			formPage.setModelDirty(false);

		} 
		catch (Exception e) {
			ErrorDialog.openError(
					getSite().getShell(),
					"Error creating nested form editor model",
					null,
					new Status(IStatus.ERROR, Activator.PLUGIN_ID,
							IStatus.ERROR, e.getMessage(), e));
		}
		try {
			formPage = new FormModelPage(formPage, model);
			addPage(formPageIndex, formPage);
			setPageText(formPageIndex, "Form");
			setActivePage(formPageIndex);

			formPage.copyFormStatus((FormModelPage) pages.get(formPageIndex + 1));
			removePage(formPageIndex + 1);

		} catch (PartInitException e) {
			ErrorDialog.openError(
					getSite().getShell(),
					"Error creating nested form editor",
					null,
					e.getStatus());
		}
		doSave(monitor);
	}


}
