/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.ui.serviceeditor.model;

import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.lang.StringEscapeUtils;
import org.spagic3.service.model.IComboProvider;
import org.spagic3.ui.serviceeditor.expression.ScrappyEvaluator;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class ServiceModelHelper extends org.spagic3.service.model.ServiceModelHelper {
			
	public ServiceModelHelper() throws Exception {
		super();
	}

	private Pattern parameterPattern = Pattern.compile("\\$\\w+");
		
	public IServiceModel createModel(String xml) throws ParserConfigurationException, SAXException, IOException {
        // Parse the XML as a W3C document.
		DocumentBuilderFactory xmlFact = DocumentBuilderFactory.newInstance();
		xmlFact.setNamespaceAware(true);
		DocumentBuilder builder = xmlFact.newDocumentBuilder();
        
		InputSource source = new InputSource(new StringReader(xml));
        Document document = builder.parse(source);

		ServiceModel model = new ServiceModel();

		model.setSpagicId(evalXPathAsString(document, "/spagic:component/@spagic.id"));
		model.setFactoryName(evalXPathAsString(document, "/spagic:component/@factory.name"));
				
		copyProperties(document, model);
		copyMapProperties(document, model);
		applyRules(model);
		
		return model;
	}
	
	private void copyProperties(Document document, IServiceModel model) {
		List<Node> propertyList = evalXPathAsNodes(document, "/spagic:component/spagic:property");
		for (Node propertyNode : propertyList) {
            final String name = evalXPathAsString(propertyNode, "@name");
            final String value = StringEscapeUtils.unescapeXml(
            		evalXPathAsString(propertyNode, "@value"));
            model.addProperty(name, value);
		}
	}
	
	private void copyMapProperties(Document document, IServiceModel model) {
		List<Node> propertyMaps = evalXPathAsNodes(document, "/spagic:component/spagic:xproperty");
		for (Node propertyMapNode : propertyMaps) {
            final String mapName = evalXPathAsString(propertyMapNode, "@name");
            model.addPropertyMap(mapName);
            List<Node> mappedPropertyList = evalXPathAsNodes(propertyMapNode, "spagic:map/spagic:entry");
            for (Node mappedPropertyNode : mappedPropertyList) {
            	final String key = evalXPathAsString(mappedPropertyNode, "spagic:string");
            	Properties properties = new Properties();
            	List<Node> keyPropertyList = evalXPathAsNodes(mappedPropertyNode, "spagic:properties/spagic:property");
            	for (Node keyPropertyNode : keyPropertyList) {

            		// XPath was failing here, and we have the
            		// attributed of the node, better to go the
            		// 'hard', low level way...
            		//
            		// Fixes and closes #598
            		NamedNodeMap attributes = keyPropertyNode.getAttributes();

            		Node temp = attributes.getNamedItem("name");
            		final String name = (temp == null) ? "" : temp.getNodeValue();

            		temp = attributes.getNamedItem("value");
            		final String value = (temp == null) ? "" : temp.getNodeValue();

                    properties.put(name, value);
            	}
            	model.addEntryToPropertyMap(mapName, key, properties);
            }
		}
	}

	public List<PropertyHelper> getDefBaseProperties(String factory) {
		List<Node> defPropertyNodes = evalXPathAsNodes(getScappyDefDocument(), "(/scrappy/definitions/def[@factory=\"" + factory + "\"]/property)");
		List<PropertyHelper> defProperties = new ArrayList<PropertyHelper>();
		for (Node defPropertyNode : defPropertyNodes) {
			defProperties.add(new PropertyHelper(defPropertyNode));
		}
		return defProperties;
	}
			
	public List<CategoryHelper> getDefinitionCategories() {
		List<Node> categoryNodes = evalXPathAsNodes(getScappyDefDocument(), "(/scrappy/connectors)" +
				" | (/scrappy/services)");
		List<CategoryHelper> categories = new ArrayList<CategoryHelper>();
		for (Node categoryNode : categoryNodes) {
			categories.add(new CategoryHelper(categoryNode));
		}
		return categories;
	}
	
	public void applyRules(IServiceModel model) {
		List<Node> defPropertyNodes = evalXPathAsNodes(getScappyDefDocument(), "(/scrappy/definitions/def[@factory=\"" + model.getFactoryName() + "\"]/property)");
		for (Node defPropertyNode : defPropertyNodes) {
			final String name = evalXPathAsString(defPropertyNode, "@name");
			if (!model.getProperties().containsKey(name)) {
	        	final String value = evalXPathAsString(defPropertyNode, "@default");
                model.addProperty(name, value);
			}
		}
		List<Node> condictionsList = evalXPathAsNodes(getScappyDefDocument(), "/scrappy/definitions/def[@factory=\"" + model.getFactoryName() + "\"]/when");
		for (Node condictionNode : condictionsList) {
			final String expr = evalXPathAsString(condictionNode, "@expr");
			final String action = evalXPathAsString(condictionNode, "@action");
			ScrappyEvaluator evaluator = new ScrappyEvaluator(expr);
        	boolean condiction = evaluator.eval(model);
        	if ("handleProperty".equals(action)) {
        		List<Node> propertyList = evalXPathAsNodes(condictionNode, "property");
        		for (Node propertyNode : propertyList) {
					final String name = evalXPathAsString(propertyNode, "@name");
					if (condiction && !model.getProperties().containsKey(name)) {
                    	final String value = evalXPathAsString(propertyNode, "@default");
	                    model.addProperty(name, value);
                    }
					if (!condiction && model.getProperties().containsKey(name)) {
						model.removeProperty(name);
					}
        		}
        	} else if ("handleKeyMap".equals(action)
        			|| "handleNumberedMap".equals(action)) {
            	final String mapName = evalXPathAsString(condictionNode, "@map");
    			final boolean mapExists = model.getMapProperties().containsKey(mapName);
    			if (!condiction) {
    				if (mapExists) {
    					model.removePropertyMap(mapName);
    				}
    			} else {
	            	final String variableName = evalXPathAsString(condictionNode, "@extractFromProperty");
	        		final String variable = model.getProperties().getProperty(variableName);
	        		Set<String> keys = new LinkedHashSet<String>();
	        		if (variable != null) {
	        			if( "handleKeyMap".equals(action)) {
		        			Matcher matcher = parameterPattern.matcher(variable);
		        			while (matcher.find()) {
		        				keys.add(matcher.group().substring(1));
		        			}
	        			} else {
	        				int number = 0;
	        				try {
	        					number = Integer.valueOf(variable);
	        				} catch (NumberFormatException nfe) {}
	        				for (int i = 1; i <= number; i++) {
	        					keys.add(new Integer(i).toString());
	        				}
	        			}
	        		}
	        		List<Node> propertyList = evalXPathAsNodes(condictionNode, "property");
	        		Map<String, Properties> oldMap = model.getMapProperties().get(mapName);
	        		model.addPropertyMap(mapName);
	        		for (String key : keys) {
	        			final boolean keyExists = mapExists && oldMap.containsKey(key);
						if (keyExists) {
							model.addEntryToPropertyMap(mapName, key, oldMap.get(key));
						} else {
	 						model.addEntryToPropertyMap(mapName, key, new Properties());
	                 		for (Node propertyNode : propertyList) {
	        					final String name = evalXPathAsString(propertyNode, "@name");
	                        	final String value = evalXPathAsString(propertyNode, "@default");
	    	                    model.getEntryForPropertyMap(mapName, key).put(name, value);
	                		}
	                    }
	        		}
    			}
        	}
        }
	}
	
	public boolean enableTarget(IServiceModel model) {
		return "true".equals(evalXPathAsString(
				getScappyDefDocument(), 
				"/scrappy/definitions/def[@factory=\"" 
					+ model.getFactoryName() 
					+ "\"]/@enableTarget"));
	}

	public List<PropertyHelper> getDefProperties(IServiceModel model) {
		List<Node> defPropertyNodes = evalXPathAsNodes(getScappyDefDocument(), "(/scrappy/definitions/def[@factory=\"" + model.getFactoryName() + "\"]/property)" +
				" | (/scrappy/definitions/def[@factory=\"" + model.getFactoryName() + "\"]/when/property)");
		List<PropertyHelper> defProperties = new ArrayList<PropertyHelper>();
		for (Node defPropertyNode : defPropertyNodes) {
			defProperties.add(new PropertyHelper(defPropertyNode));
		}
		return defProperties;
	}

	public List<PropertyHelper> getDefProperties(IServiceModel model, String category) {
		List<Node> defPropertyNodes;
		if (category == null || "".equals(category)) {
			defPropertyNodes = evalXPathAsNodes(getScappyDefDocument(), "(/scrappy/definitions/def[@factory=\"" + model.getFactoryName() + "\"]/property[not(@uicategory)])" +
					" | (/scrappy/definitions/def[@factory=\"" + model.getFactoryName() + "\"]/when/property[not(@uicategory)])");
		} else {
			defPropertyNodes = evalXPathAsNodes(getScappyDefDocument(), "(/scrappy/definitions/def[@factory=\"" + model.getFactoryName() + "\"]/property[@uicategory=\"" + category + "\"])" +
				" | (/scrappy/definitions/def[@factory=\"" + model.getFactoryName() + "\"]/when/property[@uicategory=\"" + category + "\"])");
		}
		List<PropertyHelper> defProperties = new ArrayList<PropertyHelper>();
		for (Node defPropertyNode : defPropertyNodes) {
			defProperties.add(new PropertyHelper(defPropertyNode));
		}
		return defProperties;
	}
	
	public Set<String> getDefUICategories(IServiceModel model) {
		List<Node> defCategoryNodes = evalXPathAsNodes(getScappyDefDocument(), "(/scrappy/definitions/def[@factory=\"" + model.getFactoryName() + "\"]/property/@uicategory)" +
				" | (/scrappy/definitions/def[@factory=\"" + model.getFactoryName() + "\"]/when/property/@uicategory)");
		Set<String> defCategories = new LinkedHashSet<String>();
		for (Node defCategoryNode : defCategoryNodes) {
//			defCategories.add(defCategoryNode.getStringValue());
			defCategories.add(defCategoryNode.getNodeValue());
		}
		return defCategories;
	}
	
	public List<String> getComboItems(String comboName) {
		List<Node> comboProviderNodes = evalXPathAsNodes(getScappyDefDocument(), "(/scrappy/combo-providers/combo-provider[@name=\"" + comboName + "\"])");
		String type = evalXPathAsString(comboProviderNodes.get(0), "@type");
		IComboProvider comboProvider = getComboProviderFactory().getComboProvider(type, comboProviderNodes);
		if (comboProvider != null) {
			return comboProvider.getComboItems();
		} else {
			return new ArrayList<String>();
		}
	}
	
	public List<MapPropertyHelper> getDefMapProperties(IServiceModel model) {
		List<Node> defMapPropertyNodes = evalXPathAsNodes(getScappyDefDocument(), "(/scrappy/definitions/def[@factory=\"" + model.getFactoryName() + "\"]/when[(@action=\"handleKeyMap\") or (@action=\"handleNumberedMap\")])");
		List<MapPropertyHelper> defMapProperties = new ArrayList<MapPropertyHelper>();
		for (Node defMapPropertyNode : defMapPropertyNodes) {
			defMapProperties.add(new MapPropertyHelper(defMapPropertyNode));
		}
		return defMapProperties;
	}

	public String getComponentName(IServiceModel model) {
		return getComponentName(model.getFactoryName());
	}
	
	public List<String> getEmptyMandatoryProperties(IServiceModel model) {
		List<Node> mandatoryPropertyNodes = evalXPathAsNodes(getScappyDefDocument(), "(/scrappy/definitions/def[@factory=\"" + model.getFactoryName() + "\"]/property[@mandatory=\"true\"])" +
				" | (/scrappy/definitions/def[@factory=\"" + model.getFactoryName() + "\"]/when/property[@mandatory=\"true\"])");
		List<String> emptyMandatoryPropertyNames = new ArrayList<String>();
		for (Node mandatoryPropertyNode : mandatoryPropertyNodes) {
			final PropertyHelper propertyHelper = new PropertyHelper(mandatoryPropertyNode);
			final String name = propertyHelper.getName();
			final String value = model.getProperties().getProperty(name);
			if ("".equals(value)) {
				emptyMandatoryPropertyNames.add(name);
			}
		}
		return emptyMandatoryPropertyNames;
	}

	public List<String> getEmptyMandatoryMapProperties(IServiceModel model) {
		List<Node> defMapPropertyNodes = evalXPathAsNodes(getScappyDefDocument(), "(/scrappy/definitions/def[@factory=\"" + model.getFactoryName() + "\"]/when[(@action=\"handleKeyMap\") or (@action=\"handleNumberedMap\")])");
		List<String> emptyMandatoryMapPropertyNames = new ArrayList<String>();
		for (Node defMapPropertyNode : defMapPropertyNodes) {
			MapPropertyHelper mapPropertyHelper = new MapPropertyHelper(defMapPropertyNode);
			final String mapName = mapPropertyHelper.getMapName();
			final List<PropertyHelper> propertyHelpers = mapPropertyHelper.getDefProperties();
			for (String key : model.getMapProperties().get(mapName).keySet()) {
				final Properties properties = model.getEntryForPropertyMap(mapName, key);
				for (PropertyHelper propertyHelper : propertyHelpers) {
					if (propertyHelper.isMandatory()) {
						final String name = propertyHelper.getName();
						final String value = properties.getProperty(name);
						if ("".equals(value)) {
							emptyMandatoryMapPropertyNames.add(mapName + ":" + key + ":" + name);
						}
					}
				}
			}
		}
		return emptyMandatoryMapPropertyNames;
	}

	public String asXML(IServiceModel model) {
		StringBuffer xml = new StringBuffer("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
		xml.append("<spagic:component\n");
		xml.append("\t\txmlns:spagic=\"urn:org:spagic3\"\n");
		xml.append("\t\txmlns=\"urn:org:spagic3\"\n");
		xml.append("\t\tspagic.id=\"").append(model.getSpagicId()).append("\"\n");
		xml.append("\t\tfactory.name=\"").append(model.getFactoryName()).append("\">\n");
		
		for(Object nameObj : model.getProperties().keySet()) {
			final String name = (String) nameObj;
			final String value = (String) model.getProperties().get(name);

//LB: ORIGINAL
//			if ("target".equals(name)
//					&& (value == null
//							|| "".equals(value))) {
//				continue;
//			}
			if (isValidString(value)){
				xml.append("\t<property name=\"").append(name).append("\" value=\"")
					.append(StringEscapeUtils.escapeXml(value))
					.append("\"/>\n");
			}
		}
		for (String mapName : model.getMapProperties().keySet()) {
			xml.append("\t<xproperty name=\"").append(mapName).append("\">\n");
			xml.append("\t\t<map>\n");
			for(Object keyObj : model.getMapProperties().get(mapName).keySet()) {
				final String key = (String) keyObj;
				xml.append("\t\t\t<entry>\n");
				xml.append("\t\t\t\t<string>").append(key).append("</string>\n");
				xml.append("\t\t\t\t<properties>\n");
				for (Object nameObj : model.getEntryForPropertyMap(mapName, key).keySet()) {
					final String name = (String) nameObj;
					final String value = (String) model.getEntryForPropertyMap(mapName, key).get(name);
					if (isValidString(value)){
						xml.append("\t\t\t\t\t<property name=\"").append(name).append("\" value=\"")
							.append(StringEscapeUtils.escapeXml(value))
							.append("\"/>\n");
					}
				}
				xml.append("\t\t\t\t</properties>\n");
				xml.append("\t\t\t</entry>\n");
			}
			xml.append("\t\t</map>\n");
			xml.append("\t</xproperty>\n");
		}
		xml.append("</spagic:component>");
		return xml.toString();
	}
	
	public boolean isValidString(String v){
		return v != null && v.trim().length() > 0;
	}
	public class PropertyHelper {
		
		private Node propertyDoc;
		
		public PropertyHelper(Node property) {
			this.propertyDoc = property;
		}
		
		public String getName() {
			return evalXPathAsString(propertyDoc, "@name");
		}

		public String getLabel() {
			return evalXPathAsString(propertyDoc, "@label");
		}

		public String getUICategory() {
			return evalXPathAsString(propertyDoc, "@uicategory");
		}

		public String getEditor() {
			return evalXPathAsString(propertyDoc, "@editor");
		}

		public String getCombo() {
			return evalXPathAsString(propertyDoc, "@combo");
		}

		public String getDefault() {
			return evalXPathAsString(propertyDoc, "@default");
		}
		
		public Set<String> getDroptarget() {
			final String value = evalXPathAsString(propertyDoc, "@droptarget");
			Set<String> rs = new HashSet<String>();
			for (String s : value.split(",")) {
				rs.add(s);
			}
			return rs;
		}

		public boolean refreshModel() {
			return "true".equals(evalXPathAsString(propertyDoc, "@refreshModel"));
		}

		public boolean isMandatory() {
			return "true".equals(evalXPathAsString(propertyDoc, "@mandatory"));
		}

		public boolean isEditable() {
			return !"false".equals(evalXPathAsString(propertyDoc, "@editable"));
		}

		public boolean isResource() {
			return "true".equals(evalXPathAsString(propertyDoc, "@resource"));
		}

	}

	public class MapPropertyHelper {
		
		private Node mapPropertyDoc;
		
		public MapPropertyHelper(Node mapProperty) {
			this.mapPropertyDoc = mapProperty;
		}
		
		public String getMapName() {
			return evalXPathAsString(mapPropertyDoc, "@map");
		}

		public List<PropertyHelper> getDefProperties() {
			List<Node> defPropertyNodes = evalXPathAsNodes(mapPropertyDoc, "property");
			List<PropertyHelper> defProperties = new ArrayList<PropertyHelper>();
			for (Node defPropertyNode : defPropertyNodes) {
				defProperties.add(new PropertyHelper(defPropertyNode));
			}
			return defProperties;
		}

	}
	
	public class CategoryHelper {
		
		private Node categoryDoc;
		private List<ServiceHelper> services = new ArrayList<ServiceHelper>();
		
		public CategoryHelper(Node category) {
			this.categoryDoc = category;

			List<Node> serviceNodes = evalXPathAsNodes(categoryDoc, "connector | service");
			for (Node serviceNode : serviceNodes) {
				services.add(new ServiceHelper(this, serviceNode));
			}
		}
		
		public String getName() {
			return categoryDoc.getNodeName();
		}
		
		public boolean hasServices() {
			return !services.isEmpty();
		}
		
		public List<ServiceHelper> getServices() {
			return services;
		}
	}

	public class ServiceHelper {
		
		private CategoryHelper category;
		private Node serviceDoc;
		
		public ServiceHelper(CategoryHelper category, Node service) {
			this.category = category;
			this.serviceDoc = service;
		}
		
		public String getType() {
			return serviceDoc.getNodeName();
		}

		public String getLabel() {
			return evalXPathAsString(serviceDoc, "@label");
		}

		public String getName() {
			return evalXPathAsString(serviceDoc, "@name");
		}

		public String getFactory() {
			return evalXPathAsString(serviceDoc, "@factory");
		}

		public CategoryHelper getCategory() {
			return category;
		}
	}
	
}
